@@ -20,22 +20,18 @@
 
 #include "box_data.h"
 #include "../data_structure/glomconversions.h"
+#include "../data_structure/layout/layoutitem_field.h"
+#include "../glom_python.h"
 #include "config.h"
 #include <libintl.h>
 
 Box_Data::Box_Data()
-: m_Button_Find(gettext("Find")),
+: m_Button_Find(Gtk::Stock::FIND),
   m_pDialogLayout(0)
 {
   m_bUnstoredData = false;
+   
 
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout"); //TODO: Use a generic layout dialog?
-  if(refXml)
-  {
-    refXml->get_widget_derived("window_data_layout", m_pDialogLayout);
-    if(m_pDialogLayout)
-      m_pDialogLayout->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
-  }
     
   //Connect signals:
   m_Button_Find.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data::on_Button_Find));
@@ -52,17 +48,27 @@ Glib::ustring Box_Data::get_WhereClause() const
   Glib::ustring strClause;
 
   //Look at each field entry and build e.g. 'Name = "Bob"'
-  for(guint i = 0; i < get_Entered_Field_count(); i++)
+  for(type_vecFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
   {
-    const Field& field = get_Entered_Field(i);
+    const Field& field = *iter;
     Glib::ustring strClausePart;
-
-    const Gnome::Gda::Value data = field.get_data();
+  
+    const Gnome::Gda::Value data = get_entered_field_data(field);
+    
     if(!GlomConversions::value_is_empty(data))
     {
-      Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
-      //TODO: "table.name"
-      strClausePart = fieldInfo.get_name() + " LIKE " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
+      bool use_this_field = true;
+      if(iter->get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
+      {
+        if(!data.get_bool())
+          use_this_field = false;
+      }
+
+      if(use_this_field)
+      {
+        //TODO: "table.name"
+        strClausePart = field.get_name() + " LIKE " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
+      }
     }
 
     if(!strClausePart.empty())
@@ -80,31 +86,85 @@ void Box_Data::on_Button_Find()
   signal_find.emit(get_WhereClause());
 }
 
-bool Box_Data::record_new_from_entered()
+Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value)
 {
-  //This should not be used it the table has an auto-increment primary key:
+  Field fieldPrimaryKey;
+  get_field_primary_key(fieldPrimaryKey);
+
+  const Glib::ustring primary_key_name = fieldPrimaryKey.get_name();
+ 
+  type_vecFields fieldsToAdd = m_Fields;
+
+  //Add the primary key if it is not normally shown:
+  type_vecFields::const_iterator iterFindPrimary = std::find_if(fieldsToAdd.begin(), fieldsToAdd.end(), predicate_FieldHasName<Field>(primary_key_name));
+  if(iterFindPrimary == fieldsToAdd.end())
+  {
+    fieldsToAdd.push_back(fieldPrimaryKey);
+  }
+
+  //Calculate any necessary field values and enter them:
+  for(type_vecFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
+  {
+    const Field& field = *iter;
+
+    //If the user did not enter something in this field:
+    Gnome::Gda::Value value = get_entered_field_data(field);
+    if(GlomConversions::value_is_empty(value)) //This deals with empty strings too.
+    {
+      //If the default value should be calculated, then calculate it:
+      const Glib::ustring calculation = field.get_calculation(); //TODO_Performance: Use a get_has_calculation() method.
+      if(!calculation.empty())
+      {
+        Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), calculation);
+        set_entered_field_data(field, value);
+      }
+    }
+  }
 
   //Get all entered field name/value pairs:
   Glib::ustring strNames;
   Glib::ustring strValues;
 
-  guint iCount = get_Entered_Field_count();
-  for(guint i = 0; i < iCount; i++)
-  {
-    Field field = get_Entered_Field(i);
-    Gnome::Gda::FieldAttributes fieldInfo = field.get_field_info();
-    Glib::ustring strFieldValue = field.sql(field.get_data());
+  //Avoid specifying the same field twice:
+  typedef std::map<Glib::ustring, bool> type_map_added;
+  type_map_added map_added;
 
-    if(!strFieldValue.empty())
+  for(type_vecFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
+  {
+    const Field& field = *iter;
+    const Glib::ustring field_name = field.get_name();
+
+    type_map_added::const_iterator iterFind = map_added.find(field_name);
+    if(iterFind == map_added.end()) //If it was not added already
     {
-      if(!strNames.empty())
+      Gnome::Gda::Value value;
+
+      //Use the specified (generated) primary key value, if there is one:
+      if(primary_key_name == field_name && !GlomConversions::value_is_empty(primary_key_value))
       {
-        strNames += ", ";
-        strValues += ", ";
+        value = primary_key_value;
+      }
+      else
+      {
+        if(use_entered_data || !field.get_calculation().empty()) //TODO_Performance: Use a get_has_calculation() method.
+          value = get_entered_field_data(field);
       }
 
-      strNames += field.get_field_info().get_name();;
-      strValues += strFieldValue;
+      Glib::ustring strFieldValue = field.sql(value);
+
+      if(!strFieldValue.empty())
+      {
+        if(!strNames.empty())
+        {
+          strNames += ", ";
+          strValues += ", ";
+        }
+
+        strNames += field_name;
+        strValues += strFieldValue;
+
+        map_added[field_name] = true;
+      }
     }
   }
 
@@ -115,9 +175,7 @@ bool Box_Data::record_new_from_entered()
     return Query_execute(strQuery);
   }
   else
-  {
-    return false; //failed.
-  }
+    return Glib::RefPtr<Gnome::Gda::DataModel>();
 }
 
 void Box_Data::set_unstored_data(bool bVal)
@@ -171,9 +229,54 @@ void Box_Data::on_dialog_layout_hide()
 
 Box_Data::type_vecFields Box_Data::get_fields_to_show() const
 {
-  return get_table_fields_to_show(m_strTableName);
+  if(m_strTableName.empty())
+  {
+    return type_vecFields();
+  }
+  else
+    return get_table_fields_to_show(m_strTableName);
 }
 
+void Box_Data::get_table_fields_to_show_add_group(const Glib::ustring& table_name, const type_vecFields& all_db_fields, const LayoutGroup& group, Box_Data::type_vecFields& vecFields) const
+{
+  //g_warning("Box_Data::get_table_fields_to_show_add_group(): table_name=%s, all_db_fields.size()=%d, group.name=%s", table_name.c_str(), all_db_fields.size(), group.get_name().c_str());
+
+  LayoutGroup::type_map_const_items items = group.get_items();
+  for(LayoutGroup::type_map_const_items::const_iterator iterItems = items.begin(); iterItems != items.end(); ++iterItems)
+  {
+    const LayoutItem* item = iterItems->second;
+
+    const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
+    if(field)
+    {
+      //Get the field info:
+      const Glib::ustring field_name = item->get_name();
+      type_vecFields::const_iterator iterFind = std::find_if(all_db_fields.begin(), all_db_fields.end(), predicate_FieldHasName<Field>(field_name));
+
+      //If the field does not exist anymore then we won't try to show it:
+      if(iterFind != all_db_fields.end() )
+      {
+         vecFields.push_back(*iterFind);
+      }
+    }
+    else
+    {
+      const LayoutGroup* item_group = dynamic_cast<const LayoutGroup*>(item);
+      if(item_group)
+      {
+        //Recurse:
+        get_table_fields_to_show_add_group(table_name, all_db_fields, *item_group, vecFields);
+      }
+    }      
+  }
+
+  if(vecFields.empty())
+  {
+    g_warning("Box_Data::get_table_fields_to_show_add_group(): Returning empty list.");
+  } 
+}
+
+
 Box_Data::type_vecFields Box_Data::get_table_fields_to_show(const Glib::ustring& table_name) const
 {
   //Get field definitions from the database, with corrections from the document:
@@ -191,7 +294,7 @@ Box_Data::type_vecFields Box_Data::get_table_fields_to_show(const Glib::ustring&
    
       //Start with the Primary Key as the first field:
       guint iPrimaryKey = 0;
-      bool bPrimaryKeyFound = get_field_primary_key(all_fields, iPrimaryKey);
+      bool bPrimaryKeyFound = get_field_primary_key_index(all_fields, iPrimaryKey);
       Glib::ustring primary_key_field_name;
       if(bPrimaryKeyFound)
       {
@@ -220,26 +323,17 @@ Box_Data::type_vecFields Box_Data::get_table_fields_to_show(const Glib::ustring&
         if(true) //!group.m_hidden)
         {
           //Get the fields:
-          for(LayoutGroup::type_map_items::const_iterator iterItems = group.m_map_items.begin(); iterItems != group.m_map_items.end(); ++iterItems)
-          {
-            const LayoutItem& item = iterItems->second;
-            
-            //Get the field info:
-            Glib::ustring field_name = item.m_field_name;
-            type_vecFields::const_iterator iterFind = std::find_if(all_fields.begin(), all_fields.end(), predicate_FieldHasName<Field>(field_name));
-
-            //If the field does not exist anymore then we won't try to show it:
-            if(iterFind != all_fields.end() )
-            {
-               result.push_back(*iterFind);
-            }
-          }
+          get_table_fields_to_show_add_group(table_name, all_fields, group, result);
         }
-        
       }
     }
   }
 
+  if(result.empty())
+  {
+    g_warning("Box_Data::get_table_fields_to_show_add_group(): Returning empty list.");
+  }
+  
   return result;
 }
 
@@ -324,8 +418,94 @@ Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, c
   return result;
 }
   
-
-
-
+Document_Glom::type_mapLayoutGroupSequence Box_Data::get_data_layout_groups(const Glib::ustring& layout)
+{
+  Document_Glom::type_mapLayoutGroupSequence layout_groups;
   
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  if(document)
+  {
+    if(!m_strTableName.empty())
+    {
+      //Get the layout information from the document:
+      layout_groups = document->get_data_layout_groups_plus_new_fields(layout, m_strTableName);
+
+      //Fill in the field information for the fields mentioned in the layout:
+      for(Document_Glom::type_mapLayoutGroupSequence::iterator iterGroups = layout_groups.begin(); iterGroups != layout_groups.end(); ++iterGroups)
+      {
+        fill_layout_group_field_info(iterGroups->second);
+      }
+    }
+  }
+
+  return layout_groups;
+}
+
+void Box_Data::fill_layout_group_field_info(LayoutGroup& group)
+{      
+  LayoutGroup::type_map_items items = group.get_items();
+  for(LayoutGroup::type_map_items::iterator iter = items.begin(); iter != items.end(); ++iter)
+  {
+    LayoutItem* item = iter->second;
+    LayoutItem_Field* item_field = dynamic_cast<LayoutItem_Field*>(item);
+    if(item_field) //If is a field rather than some other layout item
+    {
+      //Get the field info:
+      Field field;
+      bool found = get_fields_for_table_one_field(m_strTableName, item_field->get_name(), field);
+      if(found)
+      {
+        item_field->m_field = field; //TODO_Performance: Just use this as the output arg?
+      }
+    }
+    else
+    {
+      LayoutGroup* item_group = dynamic_cast<LayoutGroup*>(item);
+      if(item_group) //If it is a group
+      {
+        //recurse, to fill the fields info in this group:
+        fill_layout_group_field_info(*item_group);
+      }
+    }
+  }
+}
+
+//static:
+bool Box_Data::get_field_primary_key_index(const type_vecFields& fields, guint& field_column)
+{
+  //Initialize input parameter:
+  field_column = 0;
   
+  //TODO_performance: Cache the primary key?
+  guint col = 0;
+  guint cols_count = fields.size();
+  while(col < cols_count)
+  {
+    if(fields[col].get_field_info().get_primary_key())
+    {
+      field_column = col;
+      return true;
+    }
+    else
+    {
+      ++col;
+    }
+  }
+
+  return false; //Not found.
+}
+
+bool Box_Data::record_delete(const Gnome::Gda::Value& primary_key_value)
+{
+
+  Field field_primary_key;
+  bool test = get_field_primary_key(field_primary_key);
+  if(test && !GlomConversions::value_is_empty(primary_key_value))
+  {
+    return Query_execute( "DELETE FROM " + m_strTableName + " WHERE " + m_strTableName + "." + field_primary_key.get_name() + " = " + field_primary_key.sql(primary_key_value) );
+  }
+  else
+  {
+    return false; //no primary key
+  }
+}
