@@ -46,12 +46,21 @@ public:
   //where_clause.
   sigc::signal<void, Glib::ustring> signal_find;
 
+  //g++ 3.4 needs this to be public when used from Box_Data_Details. I'm not sure why. murrayc.
+  virtual void on_dialog_layout_hide();
+    
 protected:
   virtual void fill_from_database(); //override.
 
-  type_vecFields get_fields_to_show() const;
+  virtual type_vecFields get_fields_to_show() const;
   type_vecFields get_table_fields_to_show(const Glib::ustring& table_name) const;
+  void get_table_fields_to_show_add_group(const Glib::ustring& table_name, const type_vecFields& all_db_fields, const LayoutGroup& group, Box_Data::type_vecFields& vecFields) const;
 
+  /** Get the layout groups, with the Field information filled in.
+   */
+  Document_Glom::type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout);
+  void fill_layout_group_field_info(LayoutGroup& group);
+  
   typedef std::pair<Field, Relationship> type_pairFieldTrigger;
   typedef std::list<type_pairFieldTrigger> type_list_lookups;
 
@@ -63,13 +72,17 @@ protected:
   /** Get the value of the @a source_field from the @a relationship, using the @a key_value.
    */
   Gnome::Gda::Value get_lookup_value(const Relationship& relationship, const Field& source_field, const Gnome::Gda::Value & key_value);
-  
-  virtual bool record_new_from_entered(); //New record with all entered field values.
+
+  virtual bool record_delete(const Gnome::Gda::Value& primary_key_value);
+  virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); //New record with all entered field values.
   guint generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name);
 
+  virtual bool get_field_primary_key(Field& field) const = 0;
+
   //Signal handlers:
   virtual void on_Button_Find(); //only used by _Find sub-classes. Should be MI.
-  virtual void on_dialog_layout_hide();
+
+ static bool get_field_primary_key_index(const type_vecFields& fields, guint& field_column);
  
   Gtk::Button m_Button_Find; //only used by _Find sub-classes. Should be MI.
   Gtk::Label m_Label_FindStatus;
