@@ -64,6 +64,7 @@ protected:
 
   void do_lookups(const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
 
+  virtual bool get_field_primary_key(Field& field) const;
 
   //Signal handlers:
   virtual void on_button_new();
@@ -75,6 +76,7 @@ protected:
   virtual void on_button_nav_last();
 
   virtual void on_userlevel_changed(AppState::userlevels user_level); //override
+  virtual void on_flowtable_layout_changed();
   
   //Signal handler: The last 2 args are bind-ed.
   virtual void on_related_record_added(Gnome::Gda::Value key_value, Glib::ustring strFromKeyName);
@@ -84,7 +86,7 @@ protected:
 
   virtual void on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& value);
 
-   
+  Field m_field_primary_key;
   Gnome::Gda::Value m_primary_key_value;
 
   //Member widgets:
